package ru.other.company.card.service.provider;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.List;
import org.junit.BeforeClass;
import org.junit.Test;
import org.mockito.Mockito;
import org.mockito.internal.util.reflection.Whitebox;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.crystals.pos.api.plugin.card.BonusOperationResponse;
import ru.crystals.pos.api.plugin.card.BonusOperationStatus;
import ru.crystals.pos.api.plugin.card.CardSearchResponse;
import ru.crystals.pos.api.plugin.card.CardSearchResponseStatus;
import ru.crystals.pos.spi.IntegrationProperties;
import ru.crystals.pos.spi.PropertiesReader;
import ru.crystals.pos.spi.ResBundle;
import ru.crystals.pos.spi.plugin.card.CardSearchEventSource;
import ru.crystals.pos.spi.plugin.card.CardSearchRequest;
import ru.crystals.pos.spi.receipt.CardType;
import ru.crystals.pos.spi.ui.UIForms;

public class ExampleCardPluginTest {
    private static Logger logger = LoggerFactory.getLogger(ExampleCardPluginTest.class);
    private static UIForms uiMock = Mockito.mock(UIForms.class);
    private static ExampleCardPlugin plugin = new ExampleCardPlugin();

    private boolean passed;

    @BeforeClass
    public static void beforeClass() {

        ResBundle resourceMock = Mockito.mock(ResBundle.class);
        Mockito.when(resourceMock.getString(Mockito.anyString())).thenReturn("test string");

        Whitebox.setInternalState(plugin, "log", logger);
        Whitebox.setInternalState(plugin, "ui", uiMock);
        Whitebox.setInternalState(plugin, "res", resourceMock);
    }

    @Test
    public void testSearchCard() {
        PropertiesReader pluginProperties = Mockito.mock(PropertiesReader.class);
        Mockito.when(pluginProperties.getInt(Mockito.anyString(), Mockito.anyInt())).thenAnswer(invocation -> {
            String arg = (String)invocation.getArguments()[0];
            switch(arg) {
                case Constants.CARD_NO_LENGTH:
                    return 12;
                case Constants.COUPON_NO_LENGTH:
                    return 16;
            }
            return -1;
        });
        Mockito.when(pluginProperties.get(Mockito.anyString(), Mockito.anyString())).thenAnswer(invocation -> {
            String arg = (String)invocation.getArguments()[0];
            switch (arg) {
                case Constants.CARD_NO_PREFIXES:
                    return "78,79";
                case Constants.COUPON_NO_PREFIXES:
                    return "812";
            }
            return null;
        });
        IntegrationProperties properties = Mockito.mock(IntegrationProperties.class);
        Mockito.when(properties.getPluginProperties()).thenReturn(pluginProperties);
        Whitebox.setInternalState(plugin, "properties", properties);

        /*
            1. Поиск не своей карты
            2. Поиск не своего купона.
            3. Поиск карты по номеру.
            4. Поиск карты по баркоду.
            5. Поиск карты по магнитной полосе.
            6. Поиск карты по NFC.
            7. Поиск купона.
         */

        // 1. Поиск не своей карты.
        CardSearchRequestEntity request = new CardSearchRequestEntity();
        request.setSearchString("800000000005");
        request.setEventSource(CardSearchEventSource.BARCODE);

        CardSearchResponse response = plugin.searchCard(request);

        assertNotNull(response);
        assertEquals(CardSearchResponseStatus.UNKNOWN_CARD, response.getResponseStatus());

        // 2. Поиск не своего купона.
        request.setSearchString("8000000000058954");
        request.setEventSource(CardSearchEventSource.BARCODE);
        response = plugin.searchCard(request);
        assertEquals(CardSearchResponseStatus.UNKNOWN_CARD, response.getResponseStatus());

        // 3. Поиск карты по номеру.
        request.setSearchString("780000000002");
        request.setEventSource(CardSearchEventSource.NUMBER);
        response = plugin.searchCard(request);
        assertEquals(CardSearchResponseStatus.OK, response.getResponseStatus());
        assertNotNull(response.getCard());
        assertEquals("780000000002", response.getCard().getCardNumber());

        // 4. Поиск карты по баркоду
        request.setSearchString("780000000002");
        request.setEventSource(CardSearchEventSource.BARCODE);
        response = plugin.searchCard(request);
        assertEquals(CardSearchResponseStatus.OK, response.getResponseStatus());
        assertNotNull(response.getCard());
        assertEquals("780000000002", response.getCard().getCardNumber());

        // 5. Поиск карты по магнитной полосе
        request.setSearchString(null);
        request.setAdditionalData(new ArrayList<>());
        request.getAdditionalData().add("");
        request.getAdditionalData().add("780000000002");
        request.getAdditionalData().add("");
        request.getAdditionalData().add("");
        request.setEventSource(CardSearchEventSource.MAGNETIC_STRIPE);
        response = plugin.searchCard(request);
        assertEquals(CardSearchResponseStatus.OK, response.getResponseStatus());
        assertNotNull(response.getCard());
        assertEquals("780000000002", response.getCard().getCardNumber());

        // 6. Поиск карты по NFC
        request.setSearchString("780000000002");
        request.setEventSource(CardSearchEventSource.NFC);
        response = plugin.searchCard(request);
        assertEquals(CardSearchResponseStatus.UNKNOWN_CARD, response.getResponseStatus());
        assertNull(response.getCard());

        // 7. Поиск купона.
        request.setSearchString("8120000000000002");
        request.setEventSource(CardSearchEventSource.NUMBER);
        response = plugin.searchCard(request);
        assertEquals(CardSearchResponseStatus.OK, response.getResponseStatus());
        assertNotNull(response.getCard());
        assertEquals("8120000000000002", response.getCard().getCardNumber());
        assertEquals(CardType.COUPON, response.getCard().getCardType());
    }

    @Test
    public void testSearchCardByMobileNumber() {
        passed = false;
        plugin.searchCardByMobileNumber("9211266667", cardSearchResponse -> {
            passed = true;
            assertNotNull(cardSearchResponse);
            assertEquals(CardSearchResponseStatus.OK, cardSearchResponse.getResponseStatus());
            assertEquals("780000000005", cardSearchResponse.getCard().getCardNumber());
        });
        if(!passed) {
            fail("onCardSearchCompleted has not been called");
        }
        // Убедимся, что окно с спиннером вызывалось.
        Mockito.verify(uiMock, Mockito.times(1))
                .showTimingOutForm(Mockito.anyString(), Mockito.anyInt(), Mockito.anyObject());

        // Сценарий с ненайденной картой
        passed = false;
        plugin.searchCardByMobileNumber("9211264667", cardSearchResponse -> {
            passed = true;
            assertNotNull(cardSearchResponse);
            assertEquals(CardSearchResponseStatus.CARD_NOT_FOUND, cardSearchResponse.getResponseStatus());
            assertNull(cardSearchResponse.getCard());
        });
        if(!passed) {
            fail("onCardSearchCompleted has not been called");
        }
        // Убедимся, что окно с спиннером вызывалось.
        Mockito.verify(uiMock, Mockito.times(2))
                .showTimingOutForm(Mockito.anyString(), Mockito.anyInt(), Mockito.anyObject());
    }

    @Test
    public void testRollback() {
        BonusOperationResponse response = plugin.rollback("123");
        assertNotNull(response);
        assertNotNull(response.getTxId());
        assertNotEquals("", response.getTxId());
        assertEquals(BonusOperationStatus.OK, response.getResponseStatus());
    }


    class CardSearchRequestEntity implements CardSearchRequest {
        private String searchString;
        private CardSearchEventSource source;
        private List<String> additionalData;

        @Override
        public String getSearchString() {
            return searchString;
        }

        @Override
        public CardSearchEventSource getEventSource() {
            return source;
        }

        @Override
        public List<String> getAdditionalData() {
            return additionalData;
        }

        public void setSearchString(String searchString) {
            this.searchString = searchString;
        }

        public void setEventSource(CardSearchEventSource source) {
            this.source = source;
        }

        public void setAdditionalData(List<String> additionalData) {
            this.additionalData = additionalData;
        }
    }
}
