package ru.crystals.setapi.plugins;

import org.slf4j.Logger;
import ru.crystals.pos.api.ext.loyal.dto.Slip;
import ru.crystals.pos.api.ext.loyal.dto.SlipParagraph;
import ru.crystals.pos.api.ext.loyal.dto.SlipParagraphType;
import ru.crystals.pos.api.plugin.GoodsPlugin;
import ru.crystals.pos.api.plugin.goods.NewLineItem;
import ru.crystals.pos.api.receipt.MerchandiseEntity;
import ru.crystals.pos.api.ui.listener.DialogListener;
import ru.crystals.pos.api.ui.listener.InputListener;
import ru.crystals.pos.spi.IntegrationProperties;
import ru.crystals.pos.spi.ResBundle;
import ru.crystals.pos.spi.annotation.Inject;
import ru.crystals.pos.spi.annotation.POSPlugin;
import ru.crystals.pos.spi.equipment.CustomerDisplay;
import ru.crystals.pos.spi.equipment.SetApiPrinter;
import ru.crystals.pos.spi.equipment.SetApiPrinterException;
import ru.crystals.pos.spi.feedback.Feedback;
import ru.crystals.pos.spi.plugin.goods.AddForSaleRequest;
import ru.crystals.pos.spi.plugin.goods.CancelDeniedException;
import ru.crystals.pos.spi.plugin.goods.InvalidLineItemException;
import ru.crystals.pos.spi.plugin.goods.RemoveFromSaleRequest;
import ru.crystals.pos.spi.receipt.LineItem;
import ru.crystals.pos.spi.receipt.Receipt;
import ru.crystals.pos.spi.ui.DialogFormParameters;
import ru.crystals.pos.spi.ui.UIForms;

import javax.annotation.PostConstruct;
import java.math.BigDecimal;
import java.text.MessageFormat;

/**
 * Пример плагина товаров.
 * Данный демонстрационный плагин товаров симулирует процесс продажи товаров, для которых от пользователя требуется ввод номера телефона.
 */
@POSPlugin(id= GoodsPluginExample.PLUGIN_ID)
public class GoodsPluginExample implements GoodsPlugin {

    /**
     * ID плагина, который также указан в metainf.xml - SetIntegration/ExternalService/GoodsPlugin[@id]
     */
    static final String PLUGIN_ID = "ru.crystals.setapi.example.goods.plugin";

    /**
     * Значение ключа для хранения телефонного номера  позиции чека.
     */
    private static final String PHONE_NUMBER = "phone.number";

    /**
     * Маска ввода номера телефона.
     */
    private static final String PHONE_NUMBER_PATTERN = "+'7'(###)###-##-##";

    /**
     * Префикс штрихкода, по которому плагин идентифицирует товары им продаваемые.
     */
    private static final String MY_BARCODE = "0123456";

    /**
     * Логгер.
     */
    @Inject
    private Logger logger;

    /**
     * Интерфейс для асинхронной манипуляции с UI кассы.
     */
    @Inject
    private UIForms uiForms;

    /**
     * Интерфейс для доступа к файлам локализации плагина (strings_*.xml) .<br>
     * По ключу умеет возвращать локализованный текст.
     */
    @Inject
    private ResBundle res;

    /**
     * Интерфейс для получения значений настроек интеграции и плагина <br>
     * указанных в metainf.xml - SetIntegration/ExternalService/Options и SetIntegration/ExternalService/GoodsPlugin/Options<p>
     * Не рекомендуется хранить копии значений настроек в полях класса, т.к. настройки могут централизованно меняться на сервере в процессе работы касс.
     */
    @Inject
    private IntegrationProperties properties;

    /**
     * Для печати предчека
     */
    @Inject
    private SetApiPrinter printer;

    @Inject
    private CustomerDisplay customerDisplay;
    /**
     * Инициализация плагина.<br>
     * Метод вызывается один раз, после создания экземпляра класса, заполнения всех необходимых инъекций, перед первым обращением к плагину.<br>
     */
    @SuppressWarnings("unused")
    @PostConstruct
    public void init() {
        logger.info("Plugin {} loaded", PLUGIN_ID);
    }

    /**
     * Добавление позиции в чек продажи.<br>
     * Касса передает управление плагину вызовом этого метода.<br>
     * Окончанием работы плагина считается вызов одного из методов в {@code request.getCallback()}
     *
     * @param request данные запроса
     */
    @Override
    public void addForSale(AddForSaleRequest request) {
        logger.trace("Entering createLineItem({})", request);
        // Пример получения настроек внешней системы, которой принадлежит плагин.
        // Это те самые <Option/> Элемента <ExternalSystem/>, которые вы задавали в metainf.xml
        String urlString = properties.getServiceProperties().get("server.url");
        Integer timeout = properties.getServiceProperties().getInt("timeoutInMs", 5000);

        // готовим модель
        String phoneNumber = getPhoneFromReceipt(request);
        if (phoneNumber != null) {
            phoneNumber = extractPhoneDigits(phoneNumber);
        }
        SaleProcessModel stateHolder = new SaleProcessModel();
        stateHolder.setPhoneNumber(phoneNumber);
        stateHolder.setState(SaleProcessState.ENTER_PHONE_NUMBER);

        // и запускаем техпроцесс
        changeSaleProcessState(stateHolder, request);
    }


    /**
     * Сердце автомата "добавление позиции в чек продажи". Все переходы м/у состояниями техпроцесса происходят через этот контроллер (метод).
     *
     * @param stateHolder новое состояние техпроцесса, в которое надо перейти
     * @param request данные запроса
     */
    private void changeSaleProcessState(SaleProcessModel stateHolder, AddForSaleRequest request) {
        logger.trace("entering changeSaleProcessState(SaleProcessModel). the argument is: {}", stateHolder);

        if (stateHolder == null || stateHolder.getState() == null) {
            // где-то накосячили с логикой/алгоритмом
            logger.error("changeSaleProcessState: internal error! The sale process state is incorrect!");
            return;
        }

        switch (stateHolder.getState()) {
            case ENTER_PHONE_NUMBER:
                // надо отобразить окно ввода номера телефона
                showEnterPhoneNumberForm(stateHolder, request, SaleProcessState.PRINT_PRESALE_SLIP, SaleProcessState.REJECT);
                break;
            case PRINT_PRESALE_SLIP:
                // надо распечатать предчек
                printPreSaleReceipt(stateHolder, request, SaleProcessState.PRESALE_DIALOG, SaleProcessState.PRINT_PRESALE_FAILED_DIALOG);
                break;
            case PRINT_PRESALE_FAILED_DIALOG:
                // надо показать диалог: "повторить попытку печати предчека?"
                showRepeatPreSaleSlipPrintingDlg(stateHolder, request, SaleProcessState.PRESALE_DIALOG, SaleProcessState.REJECT);
                break;
            case PRESALE_DIALOG:
                // надо показать диалог: "корректна ли информация, распечатанная на предчеке?"
                showPreSaleSlipPrintedDlg(stateHolder, request, SaleProcessState.ACTIVATE, SaleProcessState.ENTER_PHONE_NUMBER);
                break;
            case ACTIVATE:
                // надо "активировать" товар
                activate(stateHolder, request, SaleProcessState.ADD, SaleProcessState.ADD);
                break;
            case ADD:
                // само добавление товара в чек продажи
                addForSale(stateHolder, request, SaleProcessState.FINISH_SUCCESS, SaleProcessState.ADDING_FAILED_MSG);
                break;
            case ADDING_FAILED_MSG:
                // надо показать сообщение с информацией об ошибке при добавлении позиции
                showAddingFailedMsg(stateHolder, request, SaleProcessState.REJECT);
                break;
            case REJECT:
                // отказ от добавления позиции в чек
                rejectSale(stateHolder, request, SaleProcessState.FINISH_FAILED, SaleProcessState.REJECT_FAILED_MSG);
                break;
            case REJECT_FAILED_MSG:
                // надо показать сообщение с информацией об ошибке при отказе от добавления позиции в чек
                rejectFailedMsg(stateHolder, request, SaleProcessState.REJECT);
                break;
            case FINISH_FAILED:
                // процесс завершен по негативному сценарию. Просто залоггируем: ничего за собой чистить не надо
                logger.error("changeSaleProcessState: failed finish");
                break;
            case FINISH_SUCCESS:
                // процесс завершен по позитивному сценарию. Просто залоггируем: ничего за собой чистить не надо
                logger.trace("changeSaleProcessState: success finish");
                break;
            default:
                // не возможно
                logger.error("changeSaleProcessState: UNKNOWN state: {}. No idea how to process", stateHolder.getState());
        }

        logger.trace("leaving changeSaleProcessState(SaleProcessModel)");
    }

    /**
     * Действия, что надо выполнить по переходу в состояние "ввод номера телефона".
     *
     * @param stateHolder модель, хранящая в том числе {@link SaleProcessModel#getPhoneNumber() номер телефона},
     *                    что должен быть введен в форме изначально (дефолтное значение)
     * @param request данные запроса
     * @param nextPositiveState следующее состояние, в которое надо перейти по успешному завершению ввода номера телефона
     * @param nextNegativeState следующее состояние, в которое надо перейти по негативному сценарию
     */
    private void showEnterPhoneNumberForm(SaleProcessModel stateHolder, AddForSaleRequest request,
                                          SaleProcessState nextPositiveState, SaleProcessState nextNegativeState) {
        logger.trace("entering showEnterPhoneNumberForm(SaleProcessModel). The argument is: stateHolder [{}]", stateHolder);
        customerDisplay.setText(res.getString("customer.display.tell.your.number"));
        uiForms.getInputForms().showPatternInputForm(
                res.getString("text.coupon"),
                res.getString("text.circulation.and.series"),
                stateHolder.getPhoneNumber(),
                res.getString("text.enter.phone.number.input.hint"),
                PHONE_NUMBER_PATTERN,
                new InputListener() {
                    @Override
                    public void eventInputComplete(String value) {
                        verifyEnteredMobileNumber(value, nextPositiveState, nextNegativeState, stateHolder, request);
                    }

                    @Override
                    public void eventCanceled() {
                        customerDisplay.setText("");
                        stateHolder.setState(nextNegativeState);
                        changeSaleProcessState(stateHolder, request);
                    }
                });

        logger.trace("leaving showEnterPhoneNumberForm(SaleProcessModel)");
    }

    private void verifyEnteredMobileNumber(String number, SaleProcessState state, SaleProcessState prev, SaleProcessModel model, AddForSaleRequest request) {
        customerDisplay.setText(res.getString("customer.display.mobile.number") + number);
        DialogFormParameters dfp = new DialogFormParameters("Номер введен верно?", "Да", "Нет");
        uiForms.showDialogForm(dfp, new DialogListener() {
            // Да
            @Override
            public void eventButton1pressed() {
                customerDisplay.setText("");
                model.setPhoneNumber(number);
                model.setState(state);
                changeSaleProcessState(model, request);
            }

            // Нет
            @Override
            public void eventButton2pressed() {
                showEnterPhoneNumberForm(model, request, state, prev);
            }

            // Отмена ввода
            @Override
            public void eventCanceled() {
                showEnterPhoneNumberForm(model, request, state, prev);
            }
        });
    }
    /**
     * Действия, что надо совершить в состоянии автомата "печать предчека".
     * @param stateHolder модель, хранящая в том числе {@link SaleProcessModel#getPhoneNumber() номер телефона},
     *                    что должен быть распечатан на предчеке
     * @param request данные запроса
     * @param nextPositiveState следующее состояние, в которое надо перейти по успешному завершению печати предчека
     * @param nextNegativeState следующее состояние, в которое надо перейти по негативному сценарию
     */
    private void printPreSaleReceipt(SaleProcessModel stateHolder, AddForSaleRequest request,
                                     SaleProcessState nextPositiveState, SaleProcessState nextNegativeState) {
        logger.trace("entering printPreSaleReceipt(SaleProcessModel). The argument is: stateHolder [{}]", stateHolder);

        String message = MessageFormat.format(res.getString("text.pre.sale.offer.slip"), stateHolder.getPhoneNumber());
        Slip slip = new Slip();
        SlipParagraph paragraph = new SlipParagraph();
        paragraph.setPayload(message);
        slip.getParagraphs().add(paragraph);

        // покажем сообщение, что печатается предчек
        logger.trace("start printing \"{}\"", message);
        uiForms.showSpinnerForm(res.getString("text.pre.sale.offer.slip.is.being.printed"));
        emulateDelay(1000L);

        try {
            printer.print(slip);

            // печать успешно завершена
            logger.trace("pre-sale slip was printed successfully");
            stateHolder.setState(nextPositiveState);
        } catch (SetApiPrinterException pe) {
            // возникла ошибка при печати
            logger.error("pre-sale slip printing failed!", pe);

            stateHolder.setErrorMessage(pe.getMessage());
            stateHolder.setState(nextNegativeState);
        }

        changeSaleProcessState(stateHolder, request);

        logger.trace("leaving printPreSaleReceipt(SaleProcessModel)");
    }

    /**
     * Действия, что надо выполнить по переходу в состояние отображения диалога "повторить печать предчека?".
     *
     * @param stateHolder модель, хранящая в том числе {@link SaleProcessModel#getErrorMessage() сообщение об ошибке},
     *                    по которой печать предчека не удалась
     * @param request данные запроса
     * @param nextPositiveState следующее состояние, в которое надо перейти по принятию позитивного решения на диалоге
     * @param nextNegativeState следующее состояние, в которое надо перейти по негативному сценарию
     */
    private void showRepeatPreSaleSlipPrintingDlg(SaleProcessModel stateHolder, AddForSaleRequest request,
                                          SaleProcessState nextPositiveState, SaleProcessState nextNegativeState) {
        logger.trace("entering showRepeatPreSaleSlipPrintingDlg(SaleProcessModel). The argument is: stateHolder [{}]", stateHolder);

        DialogFormParameters dfp = new DialogFormParameters(stateHolder.getErrorMessage(),
                res.getString("text.pre.sale.repeat"),
                res.getString("text.pre.sale.cancel"));

        uiForms.showDialogForm(dfp, new DialogListener() {
            @Override
            public void eventCanceled() {
                stateHolder.setState(nextNegativeState);
                changeSaleProcessState(stateHolder, request);
            }

            @Override
            public void eventButton1pressed() {
                stateHolder.setState(nextPositiveState);
                changeSaleProcessState(stateHolder, request);
            }

            @Override
            public void eventButton2pressed() {
                stateHolder.setState(nextNegativeState);
                changeSaleProcessState(stateHolder, request);
            }
        });

        logger.trace("leaving showRepeatPreSaleSlipPrintingDlg(SaleProcessModel)");
    }

    /**
     * Действия, что надо выполнить по переходу в состояние отображения диалога "корректна ли информация на предчеке?".
     *
     * @param stateHolder модель
     * @param request данные запроса
     * @param nextPositiveState следующее состояние, в которое надо перейти по принятию позитивного решения на диалоге
     * @param nextNegativeState следующее состояние, в которое надо перейти по негативному сценарию
     */
    private void showPreSaleSlipPrintedDlg(SaleProcessModel stateHolder, AddForSaleRequest request,
                                                  SaleProcessState nextPositiveState, SaleProcessState nextNegativeState) {
        logger.trace("entering showPreSaleSlipPrintedDlg(SaleProcessModel). The argument is: stateHolder [{}]", stateHolder);

        DialogFormParameters dfp = new DialogFormParameters(res.getString("text.pre.sale.warning"),
                res.getString("text.pre.sale.proceed.button.label"),
                res.getString("text.pre.sale.edit.phone.button.label"));

        uiForms.showDialogForm(dfp, new DialogListener() {
            @Override
            public void eventCanceled() {
                stateHolder.setState(nextNegativeState);
                changeSaleProcessState(stateHolder, request);
            }

            @Override
            public void eventButton1pressed() {
                stateHolder.setState(nextPositiveState);
                changeSaleProcessState(stateHolder, request);
            }

            @Override
            public void eventButton2pressed() {
                stateHolder.setState(nextNegativeState);
                changeSaleProcessState(stateHolder, request);
            }
        });

        logger.trace("leaving showPreSaleSlipPrintedDlg(SaleProcessModel)");
    }

    /**
     * Действия, что надо совершить в состоянии автомата "активация товара в процессинге".
     * @param stateHolder модель
     * @param request данные запроса
     * @param nextPositiveState следующее состояние, в которое надо перейти по успешному завершению активации
     * @param nextNegativeState следующее состояние, в которое надо перейти по негативному сценарию
     */
    private void activate(SaleProcessModel stateHolder, AddForSaleRequest request,
                                     SaleProcessState nextPositiveState, SaleProcessState nextNegativeState) {
        logger.trace("entering activate(SaleProcessModel). The argument is: stateHolder [{}]", stateHolder);

        // Показываем спиннер на время длительных операций.
        // Пока эта форма на экране. процесс прервать невозможно.
        uiForms.showSpinnerForm(res.getString("spinner.connection.text"));

        emulateRemoteRequest();

        // Создание новой позиции из переданного баркода.
        BigDecimal price = BigDecimal.valueOf(10000, 2);
        long qnty = 1000; // 1 шт.
        // Показано заполнение обязательных полей
        NewLineItem newLineItem = new NewLineItem("ABC012345678", request.getMerchandise().getName(), price, qnty, 0.f);
        newLineItem.setBarcode(request.getMerchandise().getBarcode());

        // Можно и так:
        // NewLineItem newLineItem = new NewLineItem(request.getMerchandise());

        // Пример заполнения расширенных свойств товара, которые будут сохранены и которые не будут сохранены в БД.
        // Это поле будет сохранено в БД, потому что оно помечено как PersistedField в metainf.xml плагина
        newLineItem.getData().put("whatever.goods.extended.attribute", "my-awesome-value");
        // Это поле не будет сохранено в БД, потому что оно не помечено как PersistedField в metainf.xml плагина
        newLineItem.getData().put("whatever.good2.extended.attribute", "attribute-value");
        // Это поле не сохраняется в БД, однако оно сохраняется на протяжении жизни позиции до её фискализации.
        // Поэтому, когда нам потребуется ещё раз продать уже иной экземпляр плагинного товара,
        // мы можем просканировать чек на предмет своих позиций и извлечь оттуда это поле зачем-то.
        newLineItem.getData().put(PHONE_NUMBER, stateHolder.getPhoneNumber());

        Slip positionSlip = new Slip();
        positionSlip.getParagraphs()
                .add(new SlipParagraph(SlipParagraphType.TEXT, "Этот слип печатается в чеке потому что вы приобрели\nтовар плагина \"" + PLUGIN_ID + "\""));
        newLineItem.getSlips().add(positionSlip);

        stateHolder.setNewLineItem(newLineItem);
        stateHolder.setState(nextPositiveState);

        changeSaleProcessState(stateHolder, request);

        logger.trace("leaving activate(SaleProcessModel)");
    }

    /**
     * Действия, что надо совершить в состоянии автомата "добавление товара в чек".
     * @param stateHolder модель, в том числе хранящая {@link SaleProcessModel#getNewLineItem() позицию}, что надо добавить в чек
     * @param request данные запроса
     * @param nextPositiveState следующее состояние, в которое надо перейти по успешному завершению добавления позиции в чек
     * @param nextNegativeState следующее состояние, в которое надо перейти по негативному сценарию
     */
    private void addForSale(SaleProcessModel stateHolder, AddForSaleRequest request,
                          SaleProcessState nextPositiveState, SaleProcessState nextNegativeState) {
        logger.trace("entering addForSale(SaleProcessModel). The argument is: stateHolder [{}]", stateHolder);

        try {
            request.getCallback().completed(stateHolder.getNewLineItem());
            stateHolder.setState(nextPositiveState);
        } catch (InvalidLineItemException e) {
            // возникла ошибка добавления позиции
            logger.error(String.format("Error send completed from addForSale newLineItem=%s", stateHolder.getNewLineItem()), e);

            stateHolder.setErrorMessage(e.getLocalizedMessage());
            stateHolder.setState(nextNegativeState);
        }

        changeSaleProcessState(stateHolder, request);

        logger.trace("leaving addForSale(SaleProcessModel)");
    }

    /**
     * Действия, что надо выполнить по переходу в состояние отображения сообщения об ошибке добавления позиции в чек.
     *
     * @param stateHolder модель, хранящая в том числе {@link SaleProcessModel#getErrorMessage() сообщение об ошибке},
     *                    что надо показать на форме
     * @param request данные запроса
     * @param nextState следующее состояние, в которое надо перейти из этого состояния
     */
    private void showAddingFailedMsg(SaleProcessModel stateHolder, AddForSaleRequest request, SaleProcessState nextState) {
        logger.trace("entering showAddingFailedMsg(SaleProcessModel). The argument is: stateHolder [{}]", stateHolder);

        // покажем форму с сообщением об ошибке
        uiForms.showErrorForm(stateHolder.getErrorMessage(), () -> { // сообщение об ошибке прочитано
            stateHolder.setState(nextState);
            changeSaleProcessState(stateHolder, request);
        });

        logger.trace("leaving showAddingFailedMsg(SaleProcessModel)");
    }

    /**
     * Действия, что надо совершить в состоянии автомата "отказ от добавления позиции в чек продажи".
     * @param stateHolder модель
     * @param request данные запроса
     * @param nextPositiveState следующее состояние, в которое надо перейти по успешному завершению операции
     * @param nextNegativeState следующее состояние, в которое надо перейти по негативному сценарию
     */
    private void rejectSale(SaleProcessModel stateHolder, AddForSaleRequest request,
                            SaleProcessState nextPositiveState, SaleProcessState nextNegativeState) {
        logger.trace("entering rejectSale(SaleProcessModel). The argument is: stateHolder [{}]", stateHolder);

        try {
            // отказываемся от добавления позиции
            request.getCallback().notCompleted();
            stateHolder.setState(nextPositiveState);
        } catch (CancelDeniedException cde) {
            // Невозможно отменить позицию, возможно не достаточно прав.
            logger.error("Error send notCompleted after error", cde);
            stateHolder.setErrorMessage(cde.getLocalizedMessage());
            stateHolder.setState(nextNegativeState);
        }

        changeSaleProcessState(stateHolder, request);

        logger.trace("leaving rejectSale(SaleProcessModel)");
    }

    /**
     * Действия, что надо выполнить по переходу в состояние отображения сообщения об ошибке отказа от добавления позиции в чек.
     *
     * @param stateHolder модель, хранящая в том числе {@link SaleProcessModel#getErrorMessage() сообщение об ошибке},
     *                    что надо показать на форме
     * @param request данные запроса
     * @param nextState следующее состояние, в которое надо перейти из этого состояния
     */
    private void rejectFailedMsg(SaleProcessModel stateHolder, AddForSaleRequest request, SaleProcessState nextState) {
        logger.trace("entering rejectFailedMsg(SaleProcessModel). The argument is: stateHolder [{}]", stateHolder);

        // покажем форму с сообщением об ошибке
        uiForms.showErrorForm(stateHolder.getErrorMessage(), () -> { // сообщение об ошибке прочитано
            stateHolder.setState(nextState);
            changeSaleProcessState(stateHolder, request);
        });

        logger.trace("leaving rejectFailedMsg(SaleProcessModel)");
    }


    public static String extractPhoneDigits(String phoneNumber) {
        return phoneNumber.replaceAll("((\\+(\\d)*[(])|([()-]))", "");
    }

    private String getPhoneFromReceipt(AddForSaleRequest request) {
        for (LineItem lineItem : request.getReceipt().getLineItems()) {
            if (PLUGIN_ID.equals(lineItem.getPluginId())) {
                String phoneNumber = lineItem.getData().get(PHONE_NUMBER);
                if (phoneNumber != null) {
                    return phoneNumber;
                }
            }
        }
        return null;
    }

    /**
     * Симуляция обращения в внешний процессинг.
     */
    void emulateRemoteRequest() {
        emulateDelay(2000);
    }

    /**
     * Эмуляция задержки
     *
     * @param delay длительность задержки, в мс
     */
    void emulateDelay(long delay) {
        try {
            Thread.sleep(delay);
        } catch(InterruptedException iex) {
            logger.error("Suddenly awoken", iex);
        }
    }

    /**
     * Удаление позиции из чека продажи
     * @param request содержит удаляемую позицию, чек и реализацию callback интерфейса
     */
    @Override
    public void removeFromSale(RemoveFromSaleRequest request) {
        logger.trace("Entering removeLineItem request={}", request);
        // Выполнение необходимых действий с позицией перед удалением позиции из чека.
        // Возвращаем управление кассе:
        request.getCallback().completed(true);
        logger.trace("Leaving removeLineItem");
    }

    /**
     * Поиск товара по штрихкоду
     * @param barcode штрихкод
     * @return товар или null
     */
    @Override
    public MerchandiseEntity findByBarcode(String barcode) {
        if (barcode.startsWith(MY_BARCODE)) {
            MerchandiseEntity result = new MerchandiseEntity();
            result.setPrice(BigDecimal.valueOf(95.96));
            result.setBarcode(barcode);
            result.setName("Товар примера плагина товаров Set API");
            // этот объект будет передан в метод addForSale в режиме продажи
            return result;
        } else {
            return null; // не известный нам штрихкод:
        }
    }

    @Override
    public Feedback eventReceiptFiscalized(Receipt receipt, boolean isCancelReceipt) {
        Feedback feedback = new Feedback("Some payload for sending to server later. IsCancel="+isCancelReceipt);
        return feedback;
    }

    @Override
    public void onRepeatSend(Feedback feedback) throws Exception {
        logger.trace("Entering onRepeatSend goodsFeedback.payload={}", feedback.getPayload());
        logger.trace("Leaving onRepeatSend");
    }
}
