package ru.crystals.pos.api.ext.loyal.dto;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import ru.crystals.pos.spi.receipt.BonusBalance;
import ru.crystals.pos.spi.receipt.Card;

/**
 * Результат применения программ лояльности к чеку
 *
 * @author aperevozchikov
 */
public class LoyaltyResult {

    /**
     * Скидки, что были даны на чек
     */
    private Collection<Discount> discounts;

    /**
     * Потолки списаний бонусов [как скидки] с бонусных карт в чеке.
     * <p>
     * Ключ - {@link Card#getId() идентификатор} бонусной карты, примененной в чеке;
     * Значение - количество бонусов, что можно списать с этой карты в данном чеке
     */
    private Map<String, BigDecimal> bonusWriteOffsLimits;

    /**
     * Списания бонусов, что были учтены при предоставлении преференций на чек.
     * <p>
     * Ключ - {@link Card#getId() идентификатор} бонусной карты, примененной в чеке;
     * Значение - количество бонусов, уже списанных с этой карты, что было учтено при предоставлении скидок на чек
     */
    private Map<String, BigDecimal> bonusesTakenIntoAccount;

    /**
     * Мапа транзакций начисления бонусных баллов на карту по результатам расчета лояльности.<br>
     * Ключом в мапе служит номер карты, на счет которой начислены бонусные баллы, значением - описание транзакции
     * начисления на карту бонусных баллов.
     */
    private Map<String, BonusAccrualResult> bonusAccrualResults;
    /**
     * Список печатных документов, которые плагин лояльности может добавить при расчете скидок.<br>
     * Будут распечатаны вместе с чеком.
     */
    private List<Slip> printDocuments;

    /**
     * Список сообщений кассиру.
     * Один элемент - одно сообщение.
     */
    private List<String> messagesToCashier;

    /**
     * Мапа расширенных атрибутов чека, которые плагин может к чеку прицепить.<br>
     * Необходима на случай, если вдруг плагинам нужно будет хранить какую-то исключительно свою информацию в чеке,
     * которая затем может выгружаться на стороне сервера в ERP или ещё зачем.
     */
    private Map<String, String> extendedAttributeMap;

    @Override
    public String toString() {
        return String.format("loy-result [discounts: %s; bonus-limits: %s]",
                getDiscounts(), getBonusWriteOffsLimits());
    }

    /**
     * Вернет скидки, что были даны на чек
     *
     * @return не {@code null}
     */
    public Collection<Discount> getDiscounts() {
        if (discounts == null) {
            discounts = new LinkedList<>();
        }
        return discounts;
    }

    /**
     * Вернет потолки списаний бонусов [как скидки] с бонусных карт в чеке
     *
     * @return не {@code null}
     * <p>
     * Ключ - {@link Card#getId() идентификатор} бонусной карты, примененной в чеке;
     * Значение - количество бонусов, что можно списать с этой карты в данном чеке
     */
    public Map<String, BigDecimal> getBonusWriteOffsLimits() {
        if (bonusWriteOffsLimits == null) {
            bonusWriteOffsLimits = new HashMap<>();
        }
        return bonusWriteOffsLimits;
    }

    /**
     * Вернет списания бонусов, что были учтены при предоставлении преференций на чек.
     *
     * @return не {@code null}
     * <p>
     * Ключ - {@link Card#getId() идентификатор} бонусной карты, примененной в чеке;
     * Значение - количество бонусов, уже списанных с этой карты, что было учтено при предоставлении скидок на чек
     * <p>
     * NOTE: если сумма {@link BonusBalance#getSumToChargeOff() уже списанных} {@link Card#getBonusBalance() бонусов с карты} не совпадет с суммой бонусов,
     * учтенных при предоставлении скидок ("преобразованных" в "денежную" скидку, например), то операция списания бонусов будет считаться невалидной и будет отменена
     * (после чего последует автоматический пересчет скидок с нулевым количеством {@link BonusBalance#getSumToChargeOff() уже списанных} бонусов)
     */
    public Map<String, BigDecimal> getBonusesTakenIntoAccount() {
        if (bonusesTakenIntoAccount == null) {
            bonusesTakenIntoAccount = new HashMap<>();
        }
        return bonusesTakenIntoAccount;
    }

    /**
     * Возвращает коллекцию печатных документов, которые плагин лояльности присовокупил к чеку при расчете скидок.
     * Каждый печатный документ (слип, листок который печатается, затем отрезается принтером) представляет
     * собой коллекцию параграфов. Параграфы могут быть текстовыми или баркодовыми, когда вместо текста распечатывается баркод формата Code39.
     * @return коллекция печатных документов или пустая коллекция, если документов нет.
     */
    public List<Slip> getPrintDocuments() {
        if(printDocuments == null) {
            printDocuments = new ArrayList<>();
        }
        return printDocuments;
    }

    /**
     * Возвращает список сообщений, которые следует отобразить кассиру после расчета скидок.<br>
     * Одному элементу списка соответствет одно сообщение.
     * @return список сообщений кассиру или пустой список, если сообщений нет.
     */
    public Collection<String> getMessagesToCashier() {
        if(messagesToCashier == null) {
            messagesToCashier = new ArrayList<>();
        }
        return messagesToCashier;
    }

    /**
     * Мапа расширенных атрибутов, которыми плагин лояльности может наградить чек.<br>
     * Заполняется плагином в случае, если он хочет добавить к чеку какие-то дополнительные параметры.
     * @return мапу расширенных атрибутов чека или пустую мапу, если таких нет.
     */
    public Map<String, String> getExtendedReceiptAttributesMap() {
        if(extendedAttributeMap == null) {
            extendedAttributeMap = new HashMap<>();
        }
        return extendedAttributeMap;
    }

    /**
     * Возвращает мапу транзакций начисления бонусных баллов на карту по результатам расчета лояльности.<br>
     * Ключом в мапе служит номер карты, на счет которой начислены бонусные баллы, значением - описание транзакции
     * начисления на карту бонусных баллов.
     * @return мапа транзакций начисления бонусных баллов или пустая мапа, если начислений не было
     */
    public Map<String, BonusAccrualResult> getBonusAccrualResults() {
        if(bonusAccrualResults == null) {
            bonusAccrualResults = new HashMap<>();
        }
        return bonusAccrualResults;
    }
}
