package ru.crystals.pos.api.plugin.payment;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * Данные выполненной оплаты.
 * <p>
 * Данный класс используется для обмена информацией с кассой о результатах выполнения оплаты.<br>
 * Касса сохраняет поля этого класса в БД.<br>
 * Поле {@code slip} используется для печати дополнительного бумажного квитка после фискализации чека.
 * <p>
 * Эти данные необходимы:<br>
 * - при аннулировании или возврате чека;<br>
 * - для экспорта данных оплат чека в ERP системы сети магазинов;<br>
 * - при визуализации чека на сервере Set Retail 10.
 * <p>
 * Информация о том, какие поля используются для визуализации и экспорта указываются в манифесте: <b>metainf.xml</b>
 */
public class Payment {

    /**
     * Сумма оплаты
     */
    private BigDecimal sum;

    /**
     * Данные транзакции выполненной оплаты для сохранения в БД &lt;ключ, значение&gt;
     */
    private Map<String, String> data = new HashMap<>();

    /**
     * Слипы для печати
     */
    private List<String> slips;

    /**
     * Сумма оплаты
     * 
     * @return сумма с двумя знаками после запятой (scale = 2)
     */
    public BigDecimal getSum() {
        return sum;
    }

    /**
     * Сумма оплаты
     * 
     * @param sum сумма с двумя знаками после запятой (scale = 2)
     */
    public void setSum(BigDecimal sum) {
        this.sum = sum;
    }

    /**
     * Данные транзакции выполненной оплаты для сохранения в БД {@code<ключ, значение>}
     * 
     * @return коллекция ключей и значений
     */
    public Map<String, String> getData() {
        return data;
    }

    /**
     * Получить значение поля по ключу из сохраненной в БД транзакции.
     * 
     * @param key ключ, по которому ищется значение
     * @return значение поля или null, если значение отсутствует
     * @see #getData()
     */
    public String getData(String key) {
        return data.get(key);
    }
    
    /**
     * Слип для печати
     * @deprecated метод устарел и не рекомендуется к использованию. Используйте метод {@link #getSlips()} для
     * получения и добавления в чек новых слипов.
     * @return текст слипа
     */
    @Deprecated
    public String getSlip() {
        if (slips == null || slips.isEmpty()) {
            return null;
        }
        return slips.get(0);
    }

    /**
     * Слип для печати
     * @deprecated метод устарел и не рекомендуется к использованию. Используйте метод {@link #getSlips()} для
     * получения и добавления в список слипов нового слипа.
     * @param slip текст слипа
     */
    @Deprecated
    public void setSlip(String slip) {
        if (this.slips == null) {
            this.slips = new LinkedList<>();
        } else {
            this.slips.clear();
        }
        this.slips.add(slip);
    }

    /**
     * Слипы для печати
     *
     * @return slips слипы
     */
    public List<String> getSlips() {
        if (slips == null) {
            slips = new LinkedList<>();
        }
        return slips;
    }
    
    /**
     * Слипы для печати
     *
     * @param slips слипы
     */
    public void setSlips(List<String> slips) {
        this.slips = slips;
    }
}
