package ru.crystals.pos.spi;

import java.util.Map;

/**
 * Интерфейс для чтения настроек {@code<ключ, значение>}.
 * <p>
 * Поскольку параметры могут быть изменены централизованно в любой момент
 * времени, не рекомендуется хранить копии их значений в классе плагина.<br>
 * 
 * <h3>Рекомендуется:</h3>
 * 
 * <pre>
 * &#064;POSPlugin(id = "foo.plugin")
 * public class FooPlugin implements PaymentPlugin {
 * 
 *     &#064;Inject
 *     private IntegrationProperties properties;
 * 
 *     &#64;Override
 *     public void doPayment(PaymentRequest request) {
 *         String ip = properties.getServiceProperties().get("processing.ip");
 *         // do something
 *     }
 * 
 * }
 * </pre>
 * 
 * <h3>Не рекомендуется:</h3>
 * 
 * <pre>
 * &#064;POSPlugin(id = "foo.plugin")
 * public class FooPlugin implements PaymentPlugin {
 * 
 *     &#064;Inject
 *     private IntegrationProperties properties;
 * 
 *     private String ip = null;
 * 
 *     &#64;Override
 *     public void doPayment(PaymentRequest request) {
 *         if (ip == null) {
 *             String ip = properties.getServiceProperties().get("processing.ip");
 *         }
 *         // do something
 *     }
 * 
 * }
 * </pre>
 * 
 */
public interface PropertiesReader {

    /**
     * Получить все настройки.
     * @return 
     *      Коллекция всех имеющихся настроек.
     */
    Map<String, String> getAllProperties();

    /**
     * Чтение символьного параметра.
     * 
     * @param key
     *            ключ параметра
     * @return Значение параметра, null если параметр отсутствует
     */
    String get(String key);

    /**
     * Чтение символьного параметра.
     * 
     * @param key
     *            ключ параметра
     * @param defaultValue
     *            значение по умолчанию
     * @return Значение параметра, если он указан, иначе <b>defaultValue</b>
     */
    String get(String key, String defaultValue);

    /**
     * Чтение целочисленного параметра.
     * 
     * @param key
     *            ключ параметра
     * @return Значение параметра, null если параметр отсутствует
     */
    Integer getInt(String key);

    /**
     * Чтение целочисленного параметра.
     * 
     * @param key
     *            ключ параметра
     * @param defaultValue
     *            значение по умолчанию
     * @return Значение параметра, если он указан, иначе <b>defaultValue</b>
     */
    int getInt(String key, int defaultValue);

    /**
     * Чтение параметра, как массива байтов.
     * 
     * @param key
     *            ключ параметра
     * @return массив байтов, null если параметр отсутствует
     */
    byte[] getBytes(String key);

    /**
     * Чтение логического параметра.
     * 
     * @param key
     *            ключ параметра
     * @return Значение параметра, если он указан, иначе <b>false</b>
     */
    boolean getBoolean(String key);

    /**
     * Чтение логического параметра.
     * 
     * @param key
     *            ключ параметра
     * @param defaultValue
     *            значение по умолчанию
     * @return Значение параметра, если он указан, иначе <b>false</b>
     */
    boolean getBoolean(String key, boolean defaultValue);    

    /**
     * Сохранить настройки.
     * @param properties - Коллекция настроек для сохранения.
     */
    void setAllProperties(Map<String, String> properties);

    /**
     * Запись символьного параметра.
     * 
     * @param key - ключ параметра
     * @param value - значение параметра
     */
    void set(String key, String value);

    /**
     * Запись целочисленного параметра.
     * 
     * @param key - ключ параметра
     * @param value - значение параметра
     */
    void setInt(String key, Integer value);

    /**
     * Запись параметра, как массива байтов.
     * 
     * @param key - ключ параметра
     * @param value - значение параметра
     */
    void setBytes(String key, byte[] value);

    /**
     * Чтение логического параметра.
     * 
     * @param key - ключ параметра
     * @param value - значение параметра
     */
    void setBoolean(String key, Boolean value);
}
