package ru.crystals.pos.spi.annotation;

import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;

/**
 * Признак основного класса плагина.
 * <p>
 * По наличию этой аннотации касса определяет, что данный класс является
 * загружаемым плагином и создает экземпляр.
 * <h3>Жизненный цикл:</h3> Singleton. Класс создается в единственном экземпляре
 * на всё время работы приложения.
 * 
 * <h3>Обязательные требования:</h3> 1. Класс должен реализовывать интерфейс
 * плагина.<br>
 * 2. Класс должен иметь конструктор по умолчанию (<i>без параметров</i>).<br>
 * 3. Параметр {@code id} должен соответствовать {@code id} указанному в
 * манифесте: <b>metainf.xml</b>
 * 
 * <h3>Пример:</h3>
 * 
 * <pre>
 * &#064;POSPlugin(id = "foo.plugin")
 * public class FooPlugin implements PaymentPlugin {
 * 
 * }
 * </pre>
 *
 */
@Target({ TYPE })
@Retention(RUNTIME)
public @interface POSPlugin {

    /**
     * Уникальный идентификатор плагина.<br>
     * Обязательный параметр.<br>
     * Должен соответствовать {@code id} указанному в манифесте:
     * <b>metainf.xml</b>
     * 
     * @return Уникальный идентификатор плагина.
     */
    String id();

}
