package ru.crystals.pos.spi.currency;

import java.math.BigDecimal;
import ru.crystals.pos.spi.Injectable;

/**
 * Реализации этого интерфейса знают как обращаться с деньгами: как их округлять при вычислении стоимости позиции или размера скидки.
 * <p>
 * <b>Далее для простоты везде этот интерфейс и его реализации будем называть "округлятором".</b>
 * <p>
 * <b>NOTE: все денежные суммы представлены в {@link BigDecimal} с двумя знаками после запятой, т.е. scale=2</b>
 *
 * @author aperevozchikov
 */
public interface ExtCurrencyHandler extends Injectable {

    /**
     * Вычислить сумму позиции и округлить в соответствиями с текущими правилами округления (т.е. просто умножение, с последующим округлением)
     *
     * @param price
     *            цена
     * @param qnty
     *            количенство в тысячных долях от единиц СИ (1000 = 1 шт.; 21458 = 21 кг 458 г)
     * @return стоимость позиции
     */
    BigDecimal getPositionSum(BigDecimal price, long qnty);

    /**
     * Вернет результат округления указанного аргумента до ближайшей <em>валидной</em> суммы денег.
     * Примеры (только для понимания того, что считается <em>валидной суммой</em> денег):
     * <ol>
     * <li> республика Беларусь (местные деньги - до 2016-07-01): валидные значения денег: 0.00, 50.00, 100.00, 200.00 и т.д. - тогда:
     * <table summary="Округление BYR">
     *  <thead>
     *      <tr><th>значение</th><th>округлится до</th></tr>
     *  </thead>
     *  <tbody>
     *      <tr><td>0.1234</td><td>0</td></tr>
     *      <tr><td>12.3456</td><td>0</td></tr>
     *      <tr><td>25.00</td><td>0</td></tr>
     *      <tr><td>75.00</td><td>100.00</td></tr>
     *      <tr><td>123.4567</td><td>100.00</td></tr>
     *  </tbody>
     * </table>
     * <li> Россия: валидные значения денег: все целые: 0.00, 0.01, 0.02, ...
     * <table summary="Округление RUB">
     *  <thead>
     *      <tr><th>значение</th><th>округлится до</th></tr>
     *  </thead>
     *  <tbody>
     *      <tr><td>0.009</td><td>0.01</td></tr>
     *      <tr><td>0.018</td><td>0.02</td></tr>
     *      <tr><td>1.2345</td><td>1.23</td></tr>
     *  </tbody>
     * </table>
     * </ol>
     *
     * @param value
     *            количество денег, чей результат округления надо вернуть
     * @return округленное значение аргумента
     */
    BigDecimal round(BigDecimal value);
}
