package ru.crystals.pos.spi.receipt;

import ru.crystals.pos.api.ext.loyal.dto.Slip;
import ru.crystals.pos.spi.annotation.POSPlugin;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Сущность, описывающая позицию в чеке.
 * <p>
 * NOTE: все денежные суммы представлены в {@link BigDecimal} с двумя знаками после запятой, т.е. scale=2
 *
 * @author aperevozchikov
 */
public interface LineItem {

    /**
     * Вернет общий размер скидок на данную позицию
     *
     * @return не {@code null}
     */
    BigDecimal getDiscountValue();

    /**
     * Вернет Товар, что продается в этой позиции
     *
     * @return не {@code null}
     */
    Merchandise getMerchandise();

    /**
     * Вернет Порядковый номер позиции в чеке
     *
     * @return Порядковый номер позиции в чеке
     */
    int getNumber();

    /**
     * Вернет Флаг-признак, указывающий можно ли давать скидку на данную позицию.
     *
     * @return {@code false}, если скидку на данную позицию давать нельзя
     */
    boolean isDiscountable();

    /**
     * Вернет Оригинальную ЦЕНУ (стоимость за единицу товара в позиции) (не будет переписываться при расчёте скидок)
     *
     * @return ЦЕНУ до расчета скидок
     */
    BigDecimal getOriginalPrice();

    /**
     * Вернет Количество товара в позиции, в тысячных долях единиц СИ (т.е. 1000 == 1 шт. (или 1 кг.), 100 == 100 гр.)
     *
     * @return количество товара в позиции, в "граммах"; строго положительное число
     */
    long getQuantity();

    /**
     * Вернет СТОИМОСТЬ позиции с учетом уже данных скидок
     *
     * @return не {@code null}
     */
    BigDecimal getSum();

    /**
     * Вернет минимально допустимую ЦЕНУ товара в данной позиции
     *
     * @return не {@code null}; не отрицательное число
     */
    BigDecimal getMinPrice();

    /**
     * Печатные документы, прикрепленные к данной позиции.
     * @return список печатных документов, прикрепленных к данной позиции или пустой список, если таких нет.
     */
    default List<Slip> getSlips() {
        return new ArrayList<>();
    }

    /**
     * Дополнительные данные товарной позиции {@code <ключ, значение>}.<br>
     * @return коллекция ключей и значений
     */
    default Map<String, String> getData() {
        return new HashMap<>();
    }

    /**
     * Вернет id типа товара (плагина) создавшего эту позицию чека.
     * @return строковый идентификатор типа товара
     * @see POSPlugin#id()
     */
    default String getPluginId() {
        return "";
    }

}
